#!/usr/bin/env python
#_*_coding:utf-8_*_

import re, sys, os
from collections import Counter
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import readFasta
import saveCode
import checkFasta

USAGE = """
USAGE:
	python EAAC.py input.fasta <sliding_window> <output>

	input.fasta:      the input protein sequence file in fasta format.
	sliding_window:   the sliding window, integer, defaule: 5
	output:           the encoding file, default: 'encodings.tsv'
	order:            the out order, select from ['alphabetically', 'polarity', 'sideChainVolume' or userDefined] 
"""

def EAAC(fastas, window=5, **kw):
	if checkFasta.checkFasta(fastas) == False:
		print('Error: for "EAAC" encoding, the input fasta sequences should be with equal length. \n\n')
		return 0

	if window < 1:
		print('Error: the sliding window should be greater than zero' + '\n\n')
		return 0

	if checkFasta.minSequenceLength(fastas) < window:
		print('Error: all the sequence length should be larger than the sliding window :' + str(window) + '\n\n')
		return 0

	AA = kw['order'] if kw['order'] != None else 'ACDEFGHIKLMNPQRSTVWY'
	#AA = 'ARNDCQEGHILKMFPSTWYV'
	encodings = []
	header = ['#']
	for w in range(1, len(fastas[0][1]) - window + 2):
		for aa in AA:
			header.append('SW.'+str(w)+'.'+aa)
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], i[1]
		code = [name]
		for j in range(len(sequence)):
			if j < len(sequence) and j + window <= len(sequence):
				count = Counter(re.sub('-', '', sequence[j:j+window]))
				for key in count:
					count[key] = count[key] / len(re.sub('-', '', sequence[j:j+window]))
				for aa in AA:
					code.append(count[aa])
		encodings.append(code)
	return encodings

if __name__ == '__main__':
	myAAorder = {
		'alphabetically': 'ACDEFGHIKLMNPQRSTVWY',
		'polarity': 'DENKRQHSGTAPYVMCWIFL',
		'sideChainVolume': 'GASDPCTNEVHQILMKRFYW',
	}
	kw = {'order': 'ACDEFGHIKLMNPQRSTVWY'}

	if len(sys.argv) == 1:
		print(USAGE)
		sys.exit(1)
	fastas = readFasta.readFasta(sys.argv[1])
	sw = int(sys.argv[2]) if len(sys.argv) >= 3 else 5
	output = sys.argv[3] if len(sys.argv) >= 4 else 'encoding.tsv'

	if len(sys.argv) >= 5:
		if sys.argv[4] in myAAorder:
			kw['order'] = myAAorder[sys.argv[4]]
		else:
			tmpOrder = re.sub('[^ACDEFGHIKLMNPQRSTVWY]', '', sys.argv[4])
			kw['order'] = tmpOrder if len(tmpOrder) == 20 else 'ACDEFGHIKLMNPQRSTVWY'

	encodings = EAAC(fastas, sw, **kw)
	saveCode.savetsv(encodings, output)




#!/usr/bin/env python
#_*_coding:utf-8_*_

import re

def CTDT(fastas, **kw):
	group1 = {
		'hydrophobicity_PRAM900101': 'RKEDQN',
		'hydrophobicity_ARGP820101': 'QSTNGDE',
		'hydrophobicity_ZIMJ680101': 'QNGSWTDERA',
		'hydrophobicity_PONP930101': 'KPDESNQT',
		'hydrophobicity_CASG920101': 'KDEQPSRNTG',
		'hydrophobicity_ENGD860101': 'RDKENQHYP',
		'hydrophobicity_FASG890101': 'KERSQD',
		'normwaalsvolume': 'GASTPDC',
		'polarity':        'LIFWCMVY',
		'polarizability':  'GASDT',
		'charge':          'KR',
		'secondarystruct': 'EALMQKRH',
		'solventaccess':   'ALFCGIVW'
	}
	group2 = {
		'hydrophobicity_PRAM900101': 'GASTPHY',
		'hydrophobicity_ARGP820101': 'RAHCKMV',
		'hydrophobicity_ZIMJ680101': 'HMCKV',
		'hydrophobicity_PONP930101': 'GRHA',
		'hydrophobicity_CASG920101': 'AHYMLV',
		'hydrophobicity_ENGD860101': 'SGTAW',
		'hydrophobicity_FASG890101': 'NTPG',
		'normwaalsvolume': 'NVEQIL',
		'polarity':        'PATGS',
		'polarizability':  'CPNVEQIL',
		'charge':          'ANCQGHILMFPSTWYV',
		'secondarystruct': 'VIYCWFT',
		'solventaccess':   'RKQEND'
	}
	group3 = {
		'hydrophobicity_PRAM900101': 'CLVIMFW',
		'hydrophobicity_ARGP820101': 'LYPFIW',
		'hydrophobicity_ZIMJ680101': 'LPFYI',
		'hydrophobicity_PONP930101': 'YMFWLCVI',
		'hydrophobicity_CASG920101': 'FIWC',
		'hydrophobicity_ENGD860101': 'CVLIMF',
		'hydrophobicity_FASG890101': 'AYHWVMFLIC',
		'normwaalsvolume': 'MHKFRYW',
		'polarity':        'HQRKNED',
		'polarizability':  'KMHFRYW',
		'charge':          'DE',
		'secondarystruct': 'GNPSD',
		'solventaccess':   'MSPTHY'
	}

	groups = [group1, group2, group3]
	property = (
	'hydrophobicity_PRAM900101', 'hydrophobicity_ARGP820101', 'hydrophobicity_ZIMJ680101', 'hydrophobicity_PONP930101',
	'hydrophobicity_CASG920101', 'hydrophobicity_ENGD860101', 'hydrophobicity_FASG890101', 'normwaalsvolume',
	'polarity', 'polarizability', 'charge', 'secondarystruct', 'solventaccess')

	encodings = []
	header = ['#']
	for p in property:
		for tr in ('Tr1221', 'Tr1331', 'Tr2332'):
			header.append(p + '.' + tr)
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		aaPair = [sequence[j:j + 2] for j in range(len(sequence) - 1)]
		for p in property:
			c1221, c1331, c2332 = 0, 0, 0
			for pair in aaPair:
				if (pair[0] in group1[p] and pair[1] in group2[p]) or (pair[0] in group2[p] and pair[1] in group1[p]):
					c1221 = c1221 + 1
					continue
				if (pair[0] in group1[p] and pair[1] in group3[p]) or (pair[0] in group3[p] and pair[1] in group1[p]):
					c1331 = c1331 + 1
					continue
				if (pair[0] in group2[p] and pair[1] in group3[p]) or (pair[0] in group3[p] and pair[1] in group2[p]):
					c2332 = c2332 + 1
			code = code + [c1221/len(aaPair), c1331/len(aaPair), c2332/len(aaPair)]
		encodings.append(code)
	return encodings


import os, platform, shutil
import pathlib
path_to_folder = pathlib.Path(__file__).parent.resolve()
class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


def find_pssm_missing_proteins(fasta_dict, pssm_dir):
    set_missing_prots = set()
    set_prots_pssm_exists = set()
    for file in os.listdir(pssm_dir):
        protein_id = file.split(".")[0]
        set_prots_pssm_exists.add(protein_id)

    for protein_id in set_prots_pssm_exists:
        file = protein_id + ".pssm"
        flag = False
        sequence = ""
        with open(pssm_dir+"/"+file, "r") as fp:
            for line in fp:
                list_line = line.strip().split()
                if len(list_line) > 0:
                    if list_line[0] == '1':
                        flag = True
                if len(list_line) == 0:
                    flag = False
                if flag:
                    sequence += list_line[1]
        if protein_id in fasta_dict:
            if sequence != fasta_dict[protein_id]:
                set_missing_prots.add(protein_id)
    set_missing_prots = set_missing_prots.union(set(fasta_dict.keys()) - set_prots_pssm_exists)
    return list(set_missing_prots)


def read_fasta_to_dict(input_dir, fasta_file, place_protein_id):
    fasta_dict = dict()
    sequence = ""
    prot_id = ""
    with open("{}/{}.fasta".format(input_dir, fasta_file), "r") as fp:
        for line in fp:
            if line[0] == '>':
                if prot_id != "" and prot_id not in fasta_dict:
                    fasta_dict[prot_id] = sequence
                prot_id = line.strip().split("|")[place_protein_id]
                if place_protein_id == 0:
                    prot_id = prot_id[1:]
                if prot_id not in fasta_dict:
                    sequence = ""
            else:
                sequence += line.strip()
        fasta_dict[prot_id] = sequence
    fp.close()
    return fasta_dict

def form_single_fasta_files(list_proteins_no_pssm, fasta_dict):
    path_single_fastas = "{}/temp_folder/single_fastas".format(path_to_folder)

    if os.path.isdir(path_single_fastas) == False:
        os.mkdir(path_single_fastas)

    for prot_id in list_proteins_no_pssm:
        fw = open("{}/{}.fasta".format(path_single_fastas, prot_id), "w")
        fw.write(">sp|{}\n{}\n".format(prot_id, fasta_dict[prot_id]))
        fw.close()


def form_missing_pssm_files(pssm_dir):
    path_single_fastas = "{}/temp_folder/single_fastas".format(path_to_folder)
    path_blast = "{}/ncbi-blast".format(path_to_folder)

    for filename in os.listdir(path_single_fastas):
        prot_id = filename.split(".")[0].strip()
        single_fasta_prot = "{}/{}".format(path_single_fastas, filename)
        pssmfile = "{}/{}.pssm".format(pssm_dir, prot_id)

        if ("Linux" in str(platform.platform())):
            os.system("{}/psiblast -db {}/uniref50_db/uniref50.blastdb -evalue 0.001 -query {} "
                      "-out_ascii_pssm {}  -out {}/outfile -num_iterations 3 -comp_based_stats 1" \
                      .format(path_blast, path_blast, single_fasta_prot, pssmfile, path_blast))
        elif ("Darwin" in str(platform.platform())):
            os.system("{}/psiblastMAC -db {}/uniref50_db/uniref50.blastdb -evalue 0.001 -query {} "
                      "-out_ascii_pssm {}  -out {}/outfile -num_iterations 3 -comp_based_stats 1" \
                      .format(path_blast, path_blast, single_fasta_prot, pssmfile, path_blast))
    try:
        shutil.rmtree(path_single_fastas)
    except OSError as e:
        print("Error: %s - %s." % (e.filename, e.strerror))
def download_extract_ncbi_blast():
    import wget
    from zipfile import ZipFile
    remote_url = 'http://slpred.kansil.org/ncbi-blast.zip'
    local_file = '{}/ncbi-blast.zip'.format(path_to_folder)
    wget.download(remote_url, local_file)
    # Create a ZipFile Object and load sample.zip in it
    with ZipFile(local_file, 'r') as zipObj:
        # Extract all the contents of zip file in current directory
        zipObj.extractall('{}'.format(path_to_folder))
        zipObj.close()
    for ncbi_file in os.listdir('{}/ncbi-blast'.format(path_to_folder)):
        path_file_ncbi = '{}/ncbi-blast/{}'.format(path_to_folder, ncbi_file)
        os.chmod(path_file_ncbi, 0o777)

def copy_pssms(fasta_dict):
    import requests
    for prot_id in fasta_dict:
        remote_url = 'https://slpred.kansil.org/swissprot_pssms/{}.pssm'.format(prot_id)
        local_file = '{}/pssm_files/{}.pssm'.format(path_to_folder, prot_id)
        data = requests.get(remote_url)
        try:
            data.raise_for_status()
        except requests.exceptions.HTTPError:
            continue
        with open(local_file, 'wb') as file:
                file.write(data.content)

def copy_form_pssm_matrices(fasta_dict):
    pssm_dir = "{}/pssm_files".format(path_to_folder)
    list_proteins_no_pssm1 = find_pssm_missing_proteins(fasta_dict, pssm_dir)
    if len(list_proteins_no_pssm1) != 0:
        copy_pssms(list_proteins_no_pssm1)
    list_proteins_no_pssm2 = find_pssm_missing_proteins(fasta_dict, pssm_dir)
    if len(list_proteins_no_pssm2) != 0:
        form_single_fasta_files(list_proteins_no_pssm2, fasta_dict)
        form_missing_pssm_files(pssm_dir)


def edit_extracted_features_POSSUM(temp_output_file, output_file, fasta_dict):
    with open(temp_output_file, 'r') as fp:
        fw = open(output_file, 'w')
        for line, prot_id in zip(fp, fasta_dict):
            fw.write('{}'.format(prot_id))
            list_line = line.strip().split(',')
            for item in list_line:
                if item == 'nan' or item == 'inf' or item == '-inf':
                    fw.write('\t0')
                else:
                    fw.write('\t{}'.format(item))
            fw.write('\n')
        fw.close()
    fp.close()
    os.remove(temp_output_file)

def edit_extracted_features_iFeature(temp_output_file, output_file, place_protein_id):
    with open(temp_output_file, 'r') as fp:
        fw = open(output_file, 'w')
        for line in fp:
            if line[0] == '#':
                continue
            else:
                line_split = line.strip().split('\t')
                protein_id = line_split[0].split('|')[place_protein_id]
                fw.write(protein_id)
                for feature in line_split[1:]:
                    fw.write('\t{}'.format(feature))
                fw.write('\n')
        fw.close()
    fp.close()
    os.remove(temp_output_file)

def download_extract_iFeature_data():
    import wget
    from zipfile import ZipFile
    remote_url = 'https://slpred.kansil.org/iFeature/data.zip'
    local_file = '{}/iFeature/data.zip'.format(path_to_folder)
    wget.download(remote_url, local_file)
    # Create a ZipFile Object and load sample.zip in it
    with ZipFile(local_file, 'r') as zipObj:
        # Extract all the contents of zip file in current directory
        zipObj.extractall('{}/iFeature/'.format(path_to_folder))
        zipObj.close()



#!/usr/bin/env python
#_*_coding:utf-8_*_

import re, sys, os, platform
import math
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import checkFasta
import readFasta
import saveCode

USAGE = """
USAGE:
	python APAAC.py input.fasta <lambda> <output>

	input.fasta:      the input protein sequence file in fasta format.
	lambda:           the lambda value, integer, defaule: 30
	output:           the encoding file, default: 'encodings.tsv'
"""


def APAAC(fastas, lambdaValue=30, w=0.05, **kw):
	if checkFasta.minSequenceLengthWithNormalAA(fastas) < lambdaValue + 1:
		print('Error: all the sequence length should be larger than the lambdaValue+1: ' + str(lambdaValue + 1) + '\n\n')
		return 0

	dataFile = re.sub('codes$', '', os.path.split(os.path.realpath(__file__))[0]) + r'\data\PAAC.txt' if platform.system() == 'Windows' else re.sub('codes$', '', os.path.split(os.path.realpath(__file__))[0]) + '/data/PAAC.txt'
	with open(dataFile) as f:
		records = f.readlines()
	AA = ''.join(records[0].rstrip().split()[1:])
	AADict = {}
	for i in range(len(AA)):
		AADict[AA[i]] = i
	AAProperty = []
	AAPropertyNames = []
	for i in range(1, len(records) - 1):
		array = records[i].rstrip().split() if records[i].rstrip() != '' else None
		AAProperty.append([float(j) for j in array[1:]])
		AAPropertyNames.append(array[0])

	AAProperty1 = []
	for i in AAProperty:
		meanI = sum(i) / 20
		fenmu = math.sqrt(sum([(j - meanI) ** 2 for j in i]) / 20)
		AAProperty1.append([(j - meanI) / fenmu for j in i])

	encodings = []
	header = ['#']
	for i in AA:
		header.append('Pc1.' + i)
	for j in range(1, lambdaValue + 1):
		for i in AAPropertyNames:
			header.append('Pc2.' + i + '.' + str(j))
	encodings.append(header)
	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		theta = []
		for n in range(1, lambdaValue + 1):
			for j in range(len(AAProperty1)):
				theta.append(sum([AAProperty1[j][AADict[sequence[k]]] * AAProperty1[j][AADict[sequence[k + n]]] for k in
								  range(len(sequence) - n)]) / (len(sequence) - n))
		myDict = {}
		for aa in AA:
			myDict[aa] = sequence.count(aa)

		code = code + [myDict[aa] / (1 + w * sum(theta)) for aa in AA]
		code = code + [w * value / (1 + w * sum(theta)) for value in theta]
		encodings.append(code)
	return encodings

if __name__ == '__main__':
	if len(sys.argv) == 1:
		print(USAGE)
		sys.exit(1)
	fastas = readFasta.readFasta(sys.argv[1])
	lambdaValue = int(sys.argv[2]) if len(sys.argv) >= 3 else 30
	output = sys.argv[3] if len(sys.argv) >= 4 else 'encoding.tsv'
	encodings = APAAC(fastas, lambdaValue)
	saveCode.savetsv(encodings, output)


#!/usr/bin/env python
#_*_coding:utf-8_*_

import re

def GDPC(fastas, **kw):
	group = {
		'alphaticr': 'GAVLMI',
		'aromatic': 'FYW',
		'postivecharger': 'KRH',
		'negativecharger': 'DE',
		'uncharger': 'STCPNQ'
	}

	groupKey = group.keys()
	baseNum = len(groupKey)
	dipeptide = [g1+'.'+g2 for g1 in groupKey for g2 in groupKey]

	index = {}
	for key in groupKey:
		for aa in group[key]:
			index[aa] = key

	encodings = []
	header = ['#'] + dipeptide
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])

		code = [name]
		myDict = {}
		for t in dipeptide:
			myDict[t] = 0

		sum = 0
		for j in range(len(sequence) - 2 + 1):
			myDict[index[sequence[j]]+'.'+index[sequence[j+1]]] = myDict[index[sequence[j]]+'.'+index[sequence[j+1]]] + 1
			sum = sum +1

		if sum == 0:
			for t in dipeptide:
				code.append(0)
		else:
			for t in dipeptide:
				code.append(myDict[t]/sum)
		encodings.append(code)

	return encodings


#!/usr/bin/env python
#_*_coding:utf-8_*_

import os

def calculateSSE(pos, end, SSE):
	newValues = SSE[pos:end]
	return [newValues.count('H')/(end-pos), newValues.count('E')/(end-pos), newValues.count('C')/(end-pos)]

def SSEC(fastas, **kw):
	ssDir = kw['path']
	if ssDir == None:
		print('Error: please specify the directory of predicted protein disorder files by "--path"')
		return 0

	encodings = []
	header = ['#', 'H', 'E', 'C']
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], i[1]
		code = [name]

		if os.path.exists(ssDir + '/' + name + '.ss2') == True:
			with open(ssDir + '/' + name + '.ss2') as f:
				records = f.readlines()[2:]
		elif os.path.exists(ssDir + '/' + name + '.spXout') == True:
			with open(ssDir + '/' + name + '.spXout') as f:
				records = f.readlines()[1:]
		else:
			print('Error: the predicted secondary structure (.ss2 or .spXout) for protein ' + name + ' does not exist.')
			return 0

		proteinSeq = ''
		SSE = []
		for line in records:
			array = line.strip().split() if line.rstrip() != '' else None
			proteinSeq = proteinSeq + array[1]
			SSE.append(array[2])

		pos = proteinSeq.find(sequence)
		if pos == -1:
			print('Warning: could not find the peptide in proteins.\n\n')
		else:
			code = code + calculateSSE(pos, pos+len(sequence), SSE)
		encodings.append(code)

	return encodings


#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os, re
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import checkFasta


def ASA(fastas, **kw):
	if checkFasta.checkFasta(fastas) == False:
		print('Error: for "ASA" encoding, the input fasta sequences should be with equal length. \n\n')
		return 0

	encodings = []
	header = ['#']
	for p in range(1, len(fastas[0][1])+1):
		header.append('ASA.F' + str(p))
	encodings.append(header)

	disDir = kw['path']
	if disDir == None:
		print('Error: please specify the directory of predicted protein ASA file by "--path"')
		return 0
	for i in fastas:
		name, sequence = i[0], i[1]
		code = [name]
		if os.path.exists(disDir + '/' + name + '.dis') == False:
			print('Error: the predicted ASA information file (.spXout) for protein ' + name + ' does not exist.')
			return 0

		with open(disDir + '/' + name + '.spXout') as f:
			records = f.readlines()[1:]

		proteinSeq = ''
		asaValue = []
		for line in records:
			array = line.strip().split() if line.strip() != '' else None
			proteinSeq = proteinSeq + array[1]
			asaValue.append(array[10])
		pos = proteinSeq.find(sequence)
		if pos == -1:
			print('Warning: could not find the peptide in proteins.\n\n')
		else:
			for p in range(pos, pos+len(sequence)):
				code.append(asaValue[p])
		encodings.append(code)

	return encodings


#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, platform, os, re
import numpy as np
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import checkFasta
import readFasta
import saveCode

USAGE = """
USAGE:
	python SOCNumber.py input.fasta <nlag> <output>

	input.fasta:      the input protein sequence file in fasta format.
	nlag:             the nlag value, integer, defaule: 30
	output:           the encoding file, default: 'encodings.tsv'
"""

def SOCNumber(fastas, nlag=30, **kw):
	if checkFasta.minSequenceLengthWithNormalAA(fastas) < nlag + 1:
		print('Error: all the sequence length should be larger than the nlag+1: ' + str(nlag + 1) + '\n\n')
		return 0

	dataFile = re.sub('codes$', '', os.path.split(os.path.realpath(__file__))[0]) + r'\data\Schneider-Wrede.txt' if platform.system() == 'Windows' else re.sub('codes$', '', os.path.split(os.path.realpath(__file__))[0]) + '/data/Schneider-Wrede.txt'
	dataFile1 = re.sub('codes$', '', os.path.split(os.path.realpath(__file__))[0]) + r'\data\Grantham.txt' if platform.system() == 'Windows' else re.sub('codes$', '', os.path.split(os.path.realpath(__file__))[0]) + '/data/Grantham.txt'
	AA = 'ACDEFGHIKLMNPQRSTVWY'
	AA1 = 'ARNDCQEGHILKMFPSTWYV'

	DictAA = {}
	for i in range(len(AA)):
		DictAA[AA[i]] = i

	DictAA1 = {}
	for i in range(len(AA1)):
		DictAA1[AA1[i]] = i

	with open(dataFile) as f:
		records = f.readlines()[1:]
	AADistance = []
	for i in records:
		array = i.rstrip().split()[1:] if i.rstrip() != '' else None
		AADistance.append(array)
	AADistance = np.array(
		[float(AADistance[i][j]) for i in range(len(AADistance)) for j in range(len(AADistance[i]))]).reshape((20, 20))

	with open(dataFile1) as f:
		records = f.readlines()[1:]
	AADistance1 = []
	for i in records:
		array = i.rstrip().split()[1:] if i.rstrip() != '' else None
		AADistance1.append(array)
	AADistance1 = np.array(
		[float(AADistance1[i][j]) for i in range(len(AADistance1)) for j in range(len(AADistance1[i]))]).reshape(
		(20, 20))

	encodings = []
	header = ['#']
	for n in range(1, nlag + 1):
		header.append('Schneider.lag' + str(n))
	for n in range(1, nlag + 1):
		header.append('gGrantham.lag' + str(n))
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		for n in range(1, nlag + 1):
			code.append(sum(
				[AADistance[DictAA[sequence[j]]][DictAA[sequence[j + n]]] ** 2 for j in range(len(sequence) - n)]) / (
						len(sequence) - n))

		for n in range(1, nlag + 1):
			code.append(sum([AADistance1[DictAA1[sequence[j]]][DictAA1[sequence[j + n]]] ** 2 for j in
							 range(len(sequence) - n)]) / (len(sequence) - n))
		encodings.append(code)
	return encodings

if __name__ == '__main__':
	if len(sys.argv) == 1:
		print(USAGE)
		sys.exit(1)
	fastas = readFasta.readFasta(sys.argv[1])
	nlag = int(sys.argv[2]) if len(sys.argv) >= 3 else 30
	output = sys.argv[3] if len(sys.argv) >= 4 else 'encoding.tsv'
	encodings = SOCNumber(fastas, nlag)
	saveCode.savetsv(encodings, output)


#!/usr/bin/env python
#_*_coding:utf-8_*_

import os, sys
import numpy as np

def readCode(file):
	encodings = []
	if not os.path.exists(file):
		print('Error: file does not exist.')
		sys.exit(1)
	with open(file) as f:
		records = f.readlines()
	for i in records:
		array = i.rstrip().split() if i.strip() != '' else None
		encodings.append(array)
	return np.array(encodings)


#!/usr/bin/env python
#_*_coding:utf-8_*_

import scipy.cluster.hierarchy as sch
import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pylab as plt

def hcluster(encodings, image='hcluster.png', **kw):
	if kw['sof'] == 'sample':
		encodings = np.array(encodings)[1:]
	else:
		encodings = np.array(encodings).T[1:]

	if len(encodings) < 3:
		return 0, 'sample number should be greater than 3.'

	#encodings = np.array(encodings)[1:]
	data = encodings[:, 1:]
	shape = data.shape
	data = np.reshape(data, shape[0] * shape[1])
	data = np.reshape([float(i) for i in data], shape)

	names = []
	for i in encodings:
		names.append(i[0])

	e = ''
	try:
		disMat = sch.distance.pdist(data, 'euclidean')
		Z = sch.linkage(disMat, method='average')
	except ValueError as e:
		print(e)
		return 0, e
	P = sch.dendrogram(Z, labels=names, leaf_rotation=270, leaf_font_size=8)
	plt.savefig(image)
	plt.close()
	cluster = sch.fcluster(Z, 1, 'inconsistent')
	res = []
	for i in range(len(data)):
		res.append([encodings[i][0], cluster[i]])
	return res, e


#!/usr/bin/env python
#_*_coding:utf-8_*_

__all__ = [
	'type1',
	'type2',
	'type3A',
	'type3B',
	'type4',
	'type5',
	'type6A',
	'type6B',
	'type6C',
	'type7',
	'type8',
	'type9',
	'type10',
	'type11',
	'type12',
	'type13',
	'type14',
	'type15',
	'type16'
	]


#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import checkFasta

def SSEB(fastas, **kw):
	if checkFasta.checkFasta(fastas) == False:
		print('Error: for "SSEB" encoding, the input fasta sequences should be with equal length. \n\n')
		return 0

	ssDir = kw['path']
	if ssDir == None:
		print('Error: please specify the directory of predicted protein disorder files by "--path"')
		return 0

	encodings = []
	header = ['#']
	for p in range(1, len(fastas[0][1])+1):
		for ss in ('H', 'E', 'C'):
				header.append('Pos'+str(p)+'.'+ss)
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], i[1]
		code = [name]
		if os.path.exists(ssDir + '/' + name + '.ss2') == True:
			with open(ssDir + '/' + name + '.ss2') as f:
				records = f.readlines()[2:]
		elif os.path.exists(ssDir + '/' + name + '.spXout') == True:
			with open(ssDir + '/' + name + '.spXout') as f:
				records = f.readlines()[1:]
		else:
			print('Error: the predicted secondary structure (.ss2 or .spXout) for protein ' + name + ' does not exist.')
			return 0

		proteinSeq = ''
		SSE = []
		myDict = {'H':[0, 0, 1], 'E':[0, 1, 0], 'C':[1, 0, 0]}
		for line in records:
			array = line.strip().split() if line.rstrip() != '' else None
			proteinSeq = proteinSeq + array[1]
			SSE.append(array[2])

		pos = proteinSeq.find(sequence)
		if pos == -1:
			print('Warning: could not find the peptide in proteins.\n\n')
		else:
			for p in range(pos, pos + len(sequence)):
				code = code + myDict[SSE[p]]
		encodings.append(code)

	return encodings


#!/usr/bin/env python
#_*_coding:utf-8_*_

import re
from collections import Counter

def AAC(fastas, **kw):
	AA = kw['order'] if kw['order'] != None else 'ACDEFGHIKLMNPQRSTVWY'
	#AA = 'ARNDCQEGHILKMFPSTWYV'
	encodings = []
	header = ['#']
	for i in AA:
		header.append(i)
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		count = Counter(sequence)
		for key in count:
			count[key] = count[key]/len(sequence)
		code = [name]
		for aa in AA:
			code.append(count[aa])
		encodings.append(code)
	return encodings


#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os, re, platform
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import checkFasta

def AAINDEX(fastas, **kw):
	if checkFasta.checkFasta(fastas) == False:
		print('Error: for "AAINDEX" encoding, the input fasta sequences should be with equal length. \n\n')
		return 0

	AA = 'ARNDCQEGHILKMFPSTWYV'

	fileAAindex = re.sub('codes$', '', os.path.split(os.path.realpath(__file__))[0]) + r'\data\AAindex.txt' if platform.system() == 'Windows' else re.sub('codes$', '', os.path.split(os.path.realpath(__file__))[0]) + '/data/AAindex.txt'
	with open(fileAAindex) as f:
		records = f.readlines()[1:]

	AAindex = []
	AAindexName = []
	for i in records:
		AAindex.append(i.rstrip().split()[1:] if i.rstrip() != '' else None)
		AAindexName.append(i.rstrip().split()[0] if i.rstrip() != '' else None)

	index = {}
	for i in range(len(AA)):
		index[AA[i]] = i

	encodings = []
	header = ['#']
	for pos in range(1, len(fastas[0][1]) + 1):
		for idName in AAindexName:
			header.append('SeqPos.' + str(pos) + '.' + idName)
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], i[1]
		code = [name]
		for aa in sequence:
			if aa == '-':
				for j in AAindex:
					code.append(0)
				continue
			for j in AAindex:
				code.append(j[index[aa]])
		encodings.append(code)

	return encodings


#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os, re
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import readFasta
import saveCode

USAGE = """
USAGE:
	python CTDCClass.py input.fasta output amino_acids_group_1 amino_acids_group_2 ... amino_acids_group_N
	
	input.fasta:                 the input protein sequence file in fasta format.	
	output:                      the encoding file.
	amino_acids_group_x          the amino acids groups.

EXAMPLE:
	python CTDCClass.py example/test-protein.txt CTDCClass.tsv RKEDQN GASTPHY CLVIMFW
"""

def Count(seq1, seq2):
	sum = 0
	for aa in seq1:
		sum = sum + seq2.count(aa)
	return sum

def CTDCClass(fastas, groups):
	encodings = []
	header = ['#']
	for g in range(len(groups)):
		header.append('g.'+str(g+1))
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		for group in groups:
			code.append(Count(group, sequence) / len(sequence))
		encodings.append(code)
	return encodings


if __name__ == '__main__':
	if len(sys.argv) < 5:
		print(USAGE)
		sys.exit(1)

	groups = sys.argv[3:]
	myStr = ''.join(groups)
	myStr = re.sub('[^ACDEFGHIKLMNPQRSTVWY]', '', myStr)
	if len(myStr) != 20 or len(set(myStr)) != 20:
		print('\nERROR: The amino acid must be no-repeat in each groups and the sum is 20!\n')
	fastas = readFasta.readFasta(sys.argv[1])
	encodings = CTDCClass(fastas, groups)
	saveCode.savetsv(encodings, sys.argv[2])


#!/usr/bin/env python
#_*_coding:utf-8_*_

import numpy as np
from sklearn.cluster import MeanShift, estimate_bandwidth

def meanshift(encodings, **kw):
	if kw['sof'] == 'sample':
		encodings = np.array(encodings)[1:]
	else:
		encodings = np.array(encodings).T[1:]

	if len(encodings) < 3:
		return 0, 'sample number should be greater than 3.'

	data = encodings[:, 1:]
	shape = data.shape
	data = np.reshape(data, shape[0] * shape[1])
	data = np.reshape([float(i) for i in data], shape)

	e = ''
	try:
		bandwidth = estimate_bandwidth(data)
		ms = MeanShift(bandwidth=bandwidth, bin_seeding=True).fit(data)
	except ValueError as e:
		print(e)
		return 0, e

	labels = ms.labels_
	cluster_centers = ms.cluster_centers_

	labels_unique = np.unique(labels)
	n_clusters_ = len(labels_unique)

	res = []
	for i in range(len(data)):
		res.append([encodings[i][0], labels[i]])
	return res, e


#!/usr/bin/env python
#_*_coding:utf-8_*_

import numpy as np
from math import sqrt

def multipl(a,b):
	sumofab=0.0
	for i in range(len(a)):
		temp=a[i]*b[i]
		sumofab+=temp
	return sumofab

def corrcoef(x,y):
	n=len(x)
	sum1=sum(x)
	sum2=sum(y)
	sumofxy=multipl(x,y)
	sumofx2 = sum([pow(i,2) for i in x])
	sumofy2 = sum([pow(j,2) for j in y])
	num=sumofxy-(float(sum1)*float(sum2)/n)
	den=sqrt((sumofx2-float(sum1**2)/n)*(sumofy2-float(sum2**2)/n))
	if den != 0:
		return num/den
	else:
		return 0

def pearsonr(encodings, labelFile):
	features = encodings[0][1:]
	encodings = np.array(encodings)[1:]
	data = encodings[:, 1:]
	shape = data.shape
	data = np.reshape(data, shape[0] * shape[1])
	data = np.reshape([float(i) for i in data], shape)

	e = ''
	if shape[0] < 5 or shape[1] < 2:
		return 0, e

	with open(labelFile) as f:
		records = f.readlines()
	myDict = {}
	try:
		for i in records:
			array = i.rstrip().split() if i.strip() != '' else None
			myDict[array[0]] = int(array[1])
	except IndexError as e:
		print(e)
		return 0, e

	labels = []
	for i in encodings:
		labels.append(myDict.get(i[0], 0))
	print(labels)

	dataShape = data.shape

	if dataShape[1] != len(features):
		print('Error: inconsistent data shape with feature number.')
		return 0, 'Error: inconsistent data shape with feature number.'
	if dataShape[0] != len(labels):
		print('Error: inconsistent data shape with sample number.')
		return 0, 'Error: inconsistent data shape with sample number.'

	myFea = {}
	for i in range(len(features)):
		array = list(data[:, i])
		try:
			myFea[features[i]] = corrcoef(array, labels)
		except (ValueError, RuntimeWarning) as e:
			return 0, e

	res = []
	res.append(['feature', 'pcc'])
	for key in sorted(myFea.items(), key=lambda item: item[1], reverse=True):
		res.append([key[0], '{0:.3f}'.format(myFea[key[0]])])
	return res, e


#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import readFasta
import saveCode
import checkFasta

USAGE = """
USAGE:
	python CKSAAP.py input.fasta <k_space> <output>

	input.fasta:      the input protein sequence file in fasta format.
	k_space:          the gap of two amino acids, integer, defaule: 5
	output:           the encoding file, default: 'encodings.tsv'
"""

def CKSAAP(fastas, gap=5, **kw):
	if gap < 0:
		print('Error: the gap should be equal or greater than zero' + '\n\n')
		return 0

	if checkFasta.minSequenceLength(fastas) < gap+2:
		print('Error: all the sequence length should be larger than the (gap value) + 2 = ' + str(gap+2) + '\n\n')
		return 0

	AA = kw['order'] if kw['order'] != None else 'ACDEFGHIKLMNPQRSTVWY'
	encodings = []
	aaPairs = []
	for aa1 in AA:
		for aa2 in AA:
			aaPairs.append(aa1 + aa2)
	header = ['#']
	for g in range(gap+1):
		for aa in aaPairs:
			header.append(aa + '.gap' + str(g))
	encodings.append(header)
	for i in fastas:
		name, sequence = i[0], i[1]
		code = [name]
		for g in range(gap+1):
			myDict = {}
			for pair in aaPairs:
				myDict[pair] = 0
			sum = 0
			for index1 in range(len(sequence)):
				index2 = index1 + g + 1
				if index1 < len(sequence) and index2 < len(sequence) and sequence[index1] in AA and sequence[index2] in AA:
					myDict[sequence[index1] + sequence[index2]] = myDict[sequence[index1] + sequence[index2]] + 1
					sum = sum + 1
			for pair in aaPairs:
				code.append(myDict[pair] / sum)
		encodings.append(code)
	return encodings

if __name__ == '__main__':
	myAAorder = {
		'alphabetically': 'ACDEFGHIKLMNPQRSTVWY',
		'polarity': 'DENKRQHSGTAPYVMCWIFL',
		'sideChainVolume': 'GASDPCTNEVHQILMKRFYW',
	}
	kw = {'order': 'ACDEFGHIKLMNPQRSTVWY'}

	if len(sys.argv) == 1:
		print(USAGE)
		sys.exit(1)
	fastas = readFasta.readFasta(sys.argv[1])
	gap = int(sys.argv[2]) if len(sys.argv) >= 3 else 5
	output = sys.argv[3] if len(sys.argv) >= 4 else 'encoding.tsv'

	if len(sys.argv) >= 5:
		if sys.argv[4] in myAAorder:
			kw['order'] = myAAorder[sys.argv[4]]
		else:
			tmpOrder = re.sub('[^ACDEFGHIKLMNPQRSTVWY]', '', sys.argv[4])
			kw['order'] = tmpOrder if len(tmpOrder) == 20 else 'ACDEFGHIKLMNPQRSTVWY'
	encodings = CKSAAP(fastas, gap, **kw)
	saveCode.savetsv(encodings, output)


#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import checkFasta

def PSSM(fastas, **kw):
	if checkFasta.checkFasta(fastas) == False:
		print('Error: for "PSSM" encoding, the input fasta sequences should be with equal length. \n\n')
		return 0

	pssmDir = kw['path']
	if pssmDir == None:
		print('Error: please specify the directory of predicted protein disorder files by "--path" \n\n')
		return 0

	AA = 'ARNDCQEGHILKMFPSTWYV'

	encodings = []
	header = ['#']
	for p in range(1, len(fastas[0][1]) + 1):
		for aa in AA:
			header.append('Pos.'+str(p) + '.' + aa)
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], i[1]
		code = [name]
		if os.path.exists(pssmDir+'/'+name+'.pssm') == False:
			print('Error: pssm prfile for protein ' + name + ' does not exist.')
			sys.exit(1)
		with open(pssmDir+'/'+name+'.pssm') as f:
			records = f.readlines()[3: -6]

		proteinSeq = ''
		pssmMatrix = []
		for line in records:
			array = line.strip().split()
			pssmMatrix.append(array[2:22])
			proteinSeq = proteinSeq + array[1]

		pos = proteinSeq.find(sequence)
		if pos == -1:
			print('Warning: could not find the peptide in proteins.\n\n')
		else:
			for p in range(pos, pos + len(sequence)):
				code = code + pssmMatrix[p]
		encodings.append(code)

	return encodings


#!/usr/bin/env python
#_*_coding:utf-8_*_

import argparse
from featureSelection import *
import sys, os, re
pPath = re.sub(r'scripts$', '', os.path.split(os.path.realpath(__file__))[0])
sys.path.append(pPath)
from clusters import readCode

if __name__ == '__main__':
	parser = argparse.ArgumentParser(usage="it's usage tip.",
									 description="feature selection")
	parser.add_argument("--file", required=True, help="input encoding file")
	parser.add_argument("--label", required=True, help="sample label")
	parser.add_argument("--type", required=True,
						choices=['CHI2', 'IG', 'MIC', 'pearsonr'], help="select cluster method")
	parser.add_argument("--out", help="output file")
	args = parser.parse_args()

	encodings = readCode.readCode(args.file)
	myFun = args.type + '.' + args.type + '(encodings, args.label)'
	print('Feature selection method: ' + args.type)
	selectedFeatures, e = eval(myFun)
	output = args.out if args.out != None else 'featureRank.txt'
	saveFeature.saveFeature(selectedFeatures, e, args.type, output)





#!/usr/bin/env python
#_*_coding:utf-8_*_

import re
import sys, os
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import readFasta
import saveCode
import checkFasta

USAGE = """
USAGE:
	python CKSAAGP.py input.fasta <k_space> <output>

	input.fasta:      the input protein sequence file in fasta format.
	k_space:          the gap of two amino acids, integer, defaule: 5
	output:           the encoding file, default: 'encodings.tsv'
"""

def generateGroupPairs(groupKey):
	gPair = {}
	for key1 in groupKey:
		for key2 in groupKey:
			gPair[key1+'.'+key2] = 0
	return gPair

def CKSAAGP(fastas, gap = 5, **kw):
	if gap < 0:
		print('Error: the gap should be equal or greater than zero' + '\n\n')
		return 0

	if checkFasta.minSequenceLength(fastas) < gap+2:
		print('Error: all the sequence length should be greater than the (gap value) + 2 = ' + str(gap+2) + '\n\n')
		return 0

	group = {
		'alphaticr': 'GAVLMI',
		'aromatic': 'FYW',
		'postivecharger': 'KRH',
		'negativecharger': 'DE',
		'uncharger': 'STCPNQ'
	}

	AA = 'ARNDCQEGHILKMFPSTWYV'

	groupKey = group.keys()

	index = {}
	for key in groupKey:
		for aa in group[key]:
			index[aa] = key

	gPairIndex = []
	for key1 in groupKey:
		for key2 in groupKey:
			gPairIndex.append(key1+'.'+key2)

	encodings = []
	header = ['#']
	for g in range(gap + 1):
		for p in gPairIndex:
			header.append(p+'.gap'+str(g))
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		for g in range(gap + 1):
			gPair = generateGroupPairs(groupKey)
			sum = 0
			for p1 in range(len(sequence)):
				p2 = p1 + g + 1
				if p2 < len(sequence) and sequence[p1] in AA and sequence[p2] in AA:
					gPair[index[sequence[p1]]+'.'+index[sequence[p2]]] = gPair[index[sequence[p1]]+'.'+index[sequence[p2]]] + 1
					sum = sum + 1

			if sum == 0:
				for gp in gPairIndex:
					code.append(0)
			else:
				for gp in gPairIndex:
					code.append(gPair[gp] / sum)

		encodings.append(code)

	return encodings

if __name__ == '__main__':
	if len(sys.argv) == 1:
		print(USAGE)
		sys.exit(1)
	fastas = readFasta.readFasta(sys.argv[1])
	gap = int(sys.argv[2]) if len(sys.argv) >= 3 else 5
	output = sys.argv[3] if len(sys.argv) >= 4 else 'encoding.tsv'
	encodings = CKSAAGP(fastas, gap)
	saveCode.savetsv(encodings, output)



#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, platform, os, re
import argparse
import numpy as np
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import checkFasta
import readFasta
import saveCode

def Geary(fastas, props=['CIDH920105', 'BHAR880101', 'CHAM820101', 'CHAM820102',
						 'CHOC760101', 'BIGC670101', 'CHAM810101', 'DAYM780201'],
				nlag = 30, **kw):
	if checkFasta.minSequenceLengthWithNormalAA(fastas) < nlag + 1:
		print('Error: all the sequence length should be larger than the nlag+1: ' + str(nlag + 1) + '\n\n')
		return 0

	AA = 'ARNDCQEGHILKMFPSTWYV'
	fileAAidx = re.sub('codes$', '', os.path.split(os.path.realpath(__file__))[0]) + r'\data\AAidx.txt' if platform.system() == 'Windows' else sys.path[0] + '/data/AAidx.txt'
	with open(fileAAidx) as f:
		records = f.readlines()[1:]
	myDict = {}
	for i in records:
		array = i.rstrip().split('\t')
		myDict[array[0]] = array[1:]

	AAidx = []
	AAidxName = []
	for i in props:
		if i in myDict:
			AAidx.append(myDict[i])
			AAidxName.append(i)
		else:
			print('"' + i + '" properties not exist.')
			return None

	AAidx1 = np.array([float(j) for i in AAidx for j in i])
	AAidx = AAidx1.reshape((len(AAidx), 20))

	propMean = np.mean(AAidx, axis=1)
	propStd = np.std(AAidx, axis=1)

	for i in range(len(AAidx)):
		for j in range(len(AAidx[i])):
			AAidx[i][j] = (AAidx[i][j] - propMean[i]) / propStd[i]

	index = {}
	for i in range(len(AA)):
		index[AA[i]] = i

	encodings = []
	header = ['#']
	for p in props:
		for n in range(1, nlag+1):
			header.append(p + '.lag' + str(n))
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		N = len(sequence)
		for prop in range(len(props)):
			xmean = sum([AAidx[prop][index[aa]] for aa in sequence]) / N
			for n in range(1, nlag + 1):
				if len(sequence) > nlag:
					# if key is '-', then the value is 0
					rn = (N-1)/(2*(N-n)) * ((sum([(AAidx[prop][index.get(sequence[j], 0)] - AAidx[prop][index.get(sequence[j + n], 0)])**2 for j in range(len(sequence)-n)])) / (sum([(AAidx[prop][index.get(sequence[j], 0)] - xmean) ** 2 for j in range(len(sequence))])))
				else:
					rn = 'NA'
				code.append(rn)
		encodings.append(code)
	return encodings

if __name__ == '__main__':
	parser = argparse.ArgumentParser(usage="it's usage tip.",
									 description="Moran descriptor")
	parser.add_argument("--file", required=True, help="input fasta file")
	parser.add_argument("--props", help="input fasta file")
	parser.add_argument("--nlag", help="input fasta file")
	parser.add_argument("--out", dest='outFile', help="the generated descriptor file")
	args = parser.parse_args()

	fastas = readFasta.readFasta(args.file)
	props = args.props.split(':') if args.props != None else ['CIDH920105', 'BHAR880101', 'CHAM820101', 'CHAM820102',
															  'CHOC760101', 'BIGC670101', 'CHAM810101', 'DAYM780201']
	nlag = int(args.nlag) if args.nlag != None else 30
	output = args.outFile if args.outFile != None else 'encoding.tsv'
	encodings = Geary(fastas, props, nlag)
	saveCode.savetsv(encodings, output)


#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Authors: Chris (chris@nohup.cc) & Young (young@nohup.cc)

import sys
import numpy as np
import math
import re
import fileinput
from featureGenerator import *

def readToMatrix(input_matrix):
    #print "start to read PSSM matrix"
    PSSM = []
    p = re.compile(r'-*[0-9]+')
    for line, strin in enumerate(input_matrix):
        if line > 2:
            str_vec = []
            overall_vec = strin.split()
            #print len(overall_vec)
            if len(overall_vec) == 0:
                break
            str_vec.extend(overall_vec[1])
            if(len(overall_vec) < 44):
                print("There is a mistake in the pssm file")
                print("Try to correct it")
                for cur_str in overall_vec[2:]:
                    str_vec.extend(p.findall(cur_str))
                    if(len(str_vec) >= 21):
                        if(len(str_vec)) >21:
                            #print len(str_vec)
                            #print str_vec
                            #print overall_vec
                            #print "Exit with an error"
                            exit(1)
                        break;
                print("Done")
            else:
                str_vec = strin.split()[1:42]
            if len(str_vec) == 0:
                break
            #str_vec_positive=map(int, str_vec[1:])
            PSSM.append(str_vec)
    fileinput.close()
    #print "finish to read PSSM matrix"
    PSSM = np.array(PSSM)
    return PSSM

def calculateDescriptors(input_matrix, algoType, argument, veriable):
    input_matrix=readToMatrix(input_matrix)

    if algoType=="aac_pssm":
        return aac_pssm(input_matrix)
    elif algoType=="d_fpssm":
        return d_fpssm(input_matrix)
    elif algoType=="smoothed_pssm":
        return smoothed_pssm(input_matrix, argument, veriable)
    elif algoType=="ab_pssm":
        return ab_pssm(input_matrix)
    elif algoType=="pssm_composition":
        return pssm_composition(input_matrix)
    elif algoType=="rpm_pssm":
        return rpm_pssm(input_matrix)
    elif algoType=="s_fpssm":
        return s_fpssm(input_matrix)
    elif algoType=="dpc_pssm":
        return dpc_pssm(input_matrix)
    elif algoType=="k_separated_bigrams_pssm":
        return k_separated_bigrams_pssm(input_matrix, argument)
    elif algoType=="tri_gram_pssm":
        return tri_gram_pssm(input_matrix)
    elif algoType=="eedp":
        return eedp(input_matrix)
    elif algoType=="tpc":
        return tpc(input_matrix)
    elif algoType=="edp":
        return edp(input_matrix)
    elif algoType=="rpssm":
        return rpssm(input_matrix)
    elif algoType=="pse_pssm":
        return pse_pssm(input_matrix, argument)
    elif algoType=="dp_pssm":
        return dp_pssm(input_matrix, argument)
    elif algoType=="pssm_ac":
        return pssm_ac(input_matrix, argument)
    elif algoType=="pssm_cc":
        return pssm_cc(input_matrix, argument)
    elif algoType=="aadp_pssm":
        return aadp_pssm(input_matrix)
    elif algoType=="aatp":
        return aatp(input_matrix)
    elif algoType=="medp":
        return medp(input_matrix)
    



#!/usr/bin/env python
#_*_coding:utf-8_*_

import re, sys, os, platform
import math
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import checkFasta
import readFasta
import saveCode

USAGE = """
USAGE:
	python PAAC.py input.fasta <lambda> <output>

	input.fasta:      the input protein sequence file in fasta format.
	lambda:           the lambda value, integer, defaule: 30
	output:           the encoding file, default: 'encodings.tsv'
"""

def Rvalue(aa1, aa2, AADict, Matrix):
	return sum([(Matrix[i][AADict[aa1]] - Matrix[i][AADict[aa2]]) ** 2 for i in range(len(Matrix))]) / len(Matrix)

def PAAC(fastas, lambdaValue=30, w=0.05, **kw):
	if checkFasta.minSequenceLengthWithNormalAA(fastas) < lambdaValue + 1:
		print('Error: all the sequence length should be larger than the lambdaValue+1: ' + str(lambdaValue + 1) + '\n\n')
		return 0

	dataFile = re.sub('codes$', '', os.path.split(os.path.realpath(__file__))[0]) + r'\data\PAAC.txt' if platform.system() == 'Windows' else re.sub('codes$', '', os.path.split(os.path.realpath(__file__))[0]) + '/data/PAAC.txt'
	with open(dataFile) as f:
		records = f.readlines()
	AA = ''.join(records[0].rstrip().split()[1:])
	AADict = {}
	for i in range(len(AA)):
		AADict[AA[i]] = i
	AAProperty = []
	AAPropertyNames = []
	for i in range(1, len(records)):
		array = records[i].rstrip().split() if records[i].rstrip() != '' else None
		AAProperty.append([float(j) for j in array[1:]])
		AAPropertyNames.append(array[0])

	AAProperty1 = []
	for i in AAProperty:
		meanI = sum(i) / 20
		fenmu = math.sqrt(sum([(j-meanI)**2 for j in i])/20)
		AAProperty1.append([(j-meanI)/fenmu for j in i])

	encodings = []
	header = ['#']
	for aa in AA:
		header.append('Xc1.' + aa)
	for n in range(1, lambdaValue + 1):
		header.append('Xc2.lambda' + str(n))
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		theta = []
		for n in range(1, lambdaValue + 1):
			theta.append(
				sum([Rvalue(sequence[j], sequence[j + n], AADict, AAProperty1) for j in range(len(sequence) - n)]) / (
				len(sequence) - n))
		myDict = {}
		for aa in AA:
			myDict[aa] = sequence.count(aa)
		code = code + [myDict[aa] / (1 + w * sum(theta)) for aa in AA]
		code = code + [(w * j) / (1 + w * sum(theta)) for j in theta]
		encodings.append(code)
	return encodings

if __name__ == '__main__':
	if len(sys.argv) == 1:
		print(USAGE)
		sys.exit(1)
	fastas = readFasta.readFasta(sys.argv[1])
	lambdaValue = int(sys.argv[2]) if len(sys.argv) >= 3 else 30
	output = sys.argv[3] if len(sys.argv) >= 4 else 'encoding.tsv'
	encodings = PAAC(fastas, lambdaValue)
	saveCode.savetsv(encodings, output)


#!/usr/bin/env python
#_*_coding:utf-8_*_

import re

def TPC(fastas, **kw):
	AA = kw['order'] if kw['order'] != None else 'ACDEFGHIKLMNPQRSTVWY'
	encodings = []
	triPeptides = [aa1 + aa2 + aa3 for aa1 in AA for aa2 in AA for aa3 in AA]
	header = ['#'] + triPeptides
	encodings.append(header)

	AADict = {}
	for i in range(len(AA)):
		AADict[AA[i]] = i

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		tmpCode = [0] * 8000
		for j in range(len(sequence) - 3 + 1):
			tmpCode[AADict[sequence[j]] * 400 + AADict[sequence[j+1]]*20 + AADict[sequence[j+2]]] = tmpCode[AADict[sequence[j]] * 400 + AADict[sequence[j+1]]*20 + AADict[sequence[j+2]]] +1
		if sum(tmpCode) != 0:
			tmpCode = [i/sum(tmpCode) for i in tmpCode]
		code = code + tmpCode
		encodings.append(code)
	return encodings



#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os, re
import pandas as pd
pPath = re.sub(r'scripts$', '', os.path.split(os.path.realpath(__file__))[0])
sys.path.append(pPath)
from clusters import readCode
import argparse
import numpy as np
from sklearn.decomposition import LatentDirichletAllocation
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt

def ldaAnalysis(encodings, labelFile, n_components = 2):
	with open(labelFile) as f:
		records = f.readlines()[1:]
	myDict = {}
	try:
		for i in records:
			array = i.rstrip().split() if i.strip() != '' else None
			myDict[array[0]] = int(array[1])
	except IndexError as e:
		return 0, e

	labels = []
	for i in encodings:
		labels.append(myDict.get(i[0], 0))

	encodings = np.array(encodings)[1:]
	data = encodings[:, 1:]
	shape = data.shape
	data = np.reshape(data, shape[0] * shape[1])
	data = np.reshape([float(i) for i in data], shape)
	newData = LatentDirichletAllocation(n_components=n_components, max_iter=5,
                                learning_method='batch',
                                learning_offset=50.,
                                random_state=0).fit_transform(data, labels)
	lda = []
	for i in range(len(data)):
		lda.append([encodings[i][0]] + list(newData[i]))
	return lda

def saveLDA(pca, file='LDA.txt'):
	with open(file, 'w') as f:
		f.write('protein\t')
		for i in range(1, len(pca[0])):
			f.write('pc.' + str(i) + '\t')
		f.write('\n')
		for i in pca:
			for j in i:
				f.write(str(j) + '\t')
			f.write('\n')
	return None

def ldaPlot(lda, label, file='lda.png'):
	lda = np.array(lda)[:]
	data = lda[:, 1:]
	shape = data.shape
	data = np.reshape(data, shape[0] * shape[1])
	data = np.reshape([float(i) for i in data], shape)

	df = pd.DataFrame({'X': data[:, 0], 'Y': data[:, 1], 'L': label})
	mySet = set(label)
	for l in mySet:
		newData = df.loc[df.loc[:, "L"] == l, :]
		plt.scatter(np.array(newData.X), np.array(newData.Y), 20, label="%s" % l)
		plt.legend(loc='best')
	plt.savefig(file)
	return None

if __name__ == '__main__':
	parser = argparse.ArgumentParser(usage="it's usage tip.", description="PCA and plot")
	parser.add_argument("--file", required=True, help="data file for LDA")
	parser.add_argument("--label", required=True, help="sample label file")
	parser.add_argument("--out", help="output file of LDA analysis.")
	parser.add_argument("--ncomponents", help="number of n components, default 2")
	args = parser.parse_args()
	n_components = int(args.ncomponents) if args.ncomponents != None else 2
	ldaFile = args.out if args.out != None else 'lda.txt'

	data = readCode.readCode(args.file)
	label = []
	if args.label != None:
		with open(args.label) as f:
			records = f.readlines()
		for line in records:
			array = line.strip().split() if line.strip() != '' else None
			label.append(array[1])

	lda = ldaAnalysis(data, args.label, n_components)
	saveLDA(lda, ldaFile)
	ldaPlot(lda, label, '%s.png' %ldaFile)






#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Authors: Chris (chris@nohup.cc) & Young (young@nohup.cc)

import sys
import numpy as np
import math
import re
import fileinput
from matrixTransformer import *

'''
Row transformation based feature encoding algorithms
'''
def aac_pssm(input_matrix):
    #print "start aac_pssm function"
    SWITCH = 0
    COUNT = 20
    seq_cn=float(np.shape(input_matrix)[0])
    aac_pssm_matrix=handleRows(input_matrix,SWITCH,COUNT)
    aac_pssm_matrix=np.array(aac_pssm_matrix)
    aac_pssm_vector=average(aac_pssm_matrix,seq_cn)
    #print "end aac_pssm function"
    return aac_pssm_vector

def d_fpssm(input_matrix):
    #print "start d_fpssm function"
    SWITCH = 1
    COUNT = 20
    d_fpssm_matrix=handleRows(input_matrix,SWITCH,COUNT)
    seqLen = float(np.shape(input_matrix)[0])
    seq_cn = 1.0
    element_max = np.amax(d_fpssm_matrix[0], axis=0)
    element_min = np.amin(d_fpssm_matrix[0], axis=0)
    d_fpssm_vector=average(d_fpssm_matrix, seq_cn)
    d_fpssm_vector=np.add(d_fpssm_vector, -element_min)
    d_fpssm_vector=np.divide(d_fpssm_vector, element_max)
    d_fpssm_vector=np.divide(d_fpssm_vector, seqLen)
    #print "end d_fpssm function"
    return d_fpssm_vector

def smoothed_pssm(input_matrix, SMOOTH, SLIDE):
    #print "start smoothed_pssm function"
    SMOOTH = 7
    SLIDE = 50
    smoothed_pssm_matrix = window(input_matrix, SMOOTH, SLIDE)
    #print "finish smoothed_pssm function"
    return smoothed_pssm_matrix

def ab_pssm(input_matrix):
    #print "start ab_pssm function"
    seq_cn=np.shape(input_matrix)[0]
    BLOCK=int(seq_cn/20)
    #print BLOCKdp_pssm
    matrix_final=[]
    for i in range(19):
        tmp=input_matrix[i*BLOCK:(i+1)*BLOCK]
        #print tmp
        matrix_final.append(aac_pssm(tmp)[0])
    tmp=input_matrix[19*BLOCK:]
    #print tmp
    matrix_final.append(aac_pssm(tmp)[0])
    ab_pssm_matrix=average(matrix_final,1.0)
    #print "finish ab_pssm function"
    return ab_pssm_matrix

def pssm_composition(input_matrix):
    #print "start pssm_composition function"
    SWITCH = 0
    COUNT = 400
    seq_cn=float(np.shape(input_matrix)[0])
    pssm_composition_matrix=handleRows(input_matrix, SWITCH, COUNT)
    pssm_composition_vector=average(pssm_composition_matrix,seq_cn)
    #print "end pssm_composition function"
    return pssm_composition_vector

def rpm_pssm(input_matrix):
    #print "start rpm_pssm function"
    SWITCH = 1
    COUNT = 400
    seq_cn=float(np.shape(input_matrix)[0])
    rpm_pssm_matrix=handleRows(input_matrix,SWITCH,COUNT)
    rpm_pssm_vector=average(rpm_pssm_matrix,seq_cn)
    #print "end rpm_pssm function"
    return rpm_pssm_vector

def s_fpssm(input_matrix):
    #print "start s_fpssm function"
    SWITCH = 2
    COUNT = 400
    seq_cn = 1
    s_fpssm_matrix=handleRows(input_matrix, SWITCH, COUNT)
    s_fpssm_matrix = np.array(s_fpssm_matrix)
    s_fpssm_matrix_shape = np.shape(s_fpssm_matrix)
    matrix_average = [(np.reshape(s_fpssm_matrix, (s_fpssm_matrix_shape[0] * s_fpssm_matrix_shape[1], )))]
    #print "end s_fpssm function"
    return matrix_average

'''
Column transformation based feature encoding algorithms
'''
def dpc_pssm(input_matrix):
    #print "start dpc_pssm function"
    PART = 0
    STEP = 1
    ID = 0
    KEY = 0
    matrix_final = preHandleColumns(input_matrix, STEP, PART, ID)
    seq_cn = float(np.shape(input_matrix)[0])
    dpc_pssm_vector = average(matrix_final, seq_cn-STEP)
    #print "end dpc_pssm function"
    return dpc_pssm_vector

def k_separated_bigrams_pssm(input_matrix,STEP):
    #print "start k_separated_bigrams_pssm function"
    PART=1
    ID=0
    KEY=0
    STEP = 1
    matrix_final = preHandleColumns(input_matrix, STEP, PART, ID)
    seq_cn = float(np.shape(input_matrix)[0])
    k_separated_bigrams_pssm_vector=average(matrix_final,10000.0)
    #print "end k_separated_bigrams_pssm function"
    return k_separated_bigrams_pssm_vector

def tri_gram_pssm(input_matrix):
    #print "start tri_gram_pssm function"
    tri_gram_pssm_matrix = handleTriColumns(input_matrix)
    #print "end tri_gram_pssm function"
    return tri_gram_pssm_matrix

def eedp(input_matrix):
    #print "start eedp function"
    STEP = 2
    PART = 0
    ID = 1
    KEY = 0
    seq_cn = float(np.shape(input_matrix)[0])
    matrix_final = preHandleColumns(input_matrix, STEP, PART, ID)
    eedp_vector = average(matrix_final, seq_cn-STEP)
    #print "end eedp function"
    return eedp_vector

def tpc(input_matrix):
    #print "start tpc function"
    PART=0
    STEP=1
    ID=0
    KEY=1
    matrix_final=preHandleColumns(input_matrix, STEP, PART, ID)
    matrix_tmp=[0.0] * 20
    matrix_tmp=np.array(matrix_tmp)
    for i in range(20):
        matrix_tmp=list(map(sum, list(zip(matrix_final[i], matrix_tmp))))
    for i in range(20):
        for j in range(20):
            matrix_final[i][j]=matrix_final[i][j]/matrix_tmp[j]
    tpc_vector = average(matrix_final, 1.0)
    #print "end tpc function"
    return tpc_vector

'''
Mixture of row and column transformation based feature encoding algorithms
'''
def edp(input_matrix):
    #print "start edp function"
    STEP=2
    PART=0
    ID=1
    edp_matrix=[ [0.0] * 20 ] * 1
    edp_matrix=np.array(edp_matrix)
    seq_cn=float(np.shape(input_matrix)[0])
    output_matrix=preHandleColumns(input_matrix,STEP,PART,ID)
    output_matrix=np.array(output_matrix)
    for i in range(20):
        edp_matrix[0]=list(map(sum, list(zip(output_matrix[i], edp_matrix[0]))))
    edp_matrix=np.divide(edp_matrix, (seq_cn-STEP)*20.0)
    #print "end edp function"
    return edp_matrix

def rpssm(input_matrix):
    #print "start rpssm function"
    rpssm_matrix=handleMixed3(input_matrix)
    #print "finish rpssm function"
    return rpssm_matrix

def pse_pssm(input_matrix, ALPHA):
    #print "start pse_pssm function"
    ALPHA=1
    pse_pssm_matrix=handleMixed(input_matrix,ALPHA)
    #print "end pse_pssm function"
    return pse_pssm_matrix

def dp_pssm(input_matrix, ALPHA):
    #print "start dp_pssm function"
    ALPHA=5
    dp_pssm_matrix=handleMixed2(input_matrix,ALPHA)
    #print "end dp_pssm function"
    return dp_pssm_matrix

def pssm_ac(input_matrix, GROUP):
    #print "start pssm_ac function"
    ID=0
    GROUP = 10
    pssm_ac_matrix=correlation(input_matrix, ID, GROUP)
    #print "end pssm_ac function"
    return pssm_ac_matrix

def pssm_cc(input_matrix, GROUP):
    #print "start pssm_cc function"
    ID=1
    GROUP = 10
    pssm_cc_matrix=correlation(input_matrix, ID, GROUP)

    #print "finish pssm_cc function"
    return pssm_cc_matrix
'''
Feature encoding algorithms that simply combine the above single feature
'''
def aadp_pssm(input_matrix):
    aac_pssm_matrix=aac_pssm(input_matrix)
    dpc_pssm_matrix=dpc_pssm(input_matrix)
    aac_pssm_matrix=np.array(aac_pssm_matrix)
    dpc_pssm_matrix=np.array(dpc_pssm_matrix)
    aadp_pssm_matrix=np.hstack((aac_pssm_matrix, dpc_pssm_matrix))
    #print np.shape(aadp_pssm_matrix)
    return aadp_pssm_matrix

def aatp(input_matrix):
    aac_pssm_matrix=aac_pssm(input_matrix)
    tpc_matrix=tpc(input_matrix)
    aac_pssm_matrix=np.array(aac_pssm_matrix)
    tpc_matrix=np.array(tpc_matrix)
    aatp_matrix=np.hstack((aac_pssm_matrix,tpc_matrix))
    #print np.shape(aatp_matrix)
    return aatp_matrix

def medp(input_matrix):
    edp_matrix=edp(input_matrix)
    eedp_matrix=eedp(input_matrix)
    edp_matrix=np.array(edp_matrix)
    eedp_matrix=np.array(eedp_matrix)
    #print edp_matrix
    #print eedp_matrix
    medp_matrix=np.hstack((edp_matrix, eedp_matrix))
    #print medp_matrix
    #print np.shape(medp_matrix)
    return medp_matrix



#!/usr/bin/env python
#_*_coding:utf-8_*_

def saveFeature(feature, e, method, file='featureRank.txt'):
	with open(file, 'w') as f:
		if feature == 0:
			f.write(str(e))
		else:
			f.write('# Feature selection method: %s\n' %method)
			f.write('# The features were ranked according to their importance, the topper the more important the feature is\n');
			f.write('=======================\n')
			for i in feature:
				f.write(i[0] + '\t' + str(i[1]) + '\n')
	return None


#!/usr/bin/env python
#_*_coding:utf-8_*_

import re

def checkFasta(fastas):
	status = True
	lenList = set()
	for i in fastas:
		lenList.add(len(i[1]))
	if len(lenList) == 1:
		return True
	else:
		return False

def minSequenceLength(fastas):
	minLen = 10000
	for i in fastas:
		if minLen > len(i[1]):
			minLen = len(i[1])
	return minLen

def minSequenceLengthWithNormalAA(fastas):
	minLen = 10000
	for i in fastas:
		if minLen > len(re.sub('-', '', i[1])):
			minLen = len(re.sub('-', '', i[1]))
	return minLen


#!/usr/bin/env python
#_*_coding:utf-8_*_

import re
import math

def DDE(fastas, **kw):
	AA = kw['order'] if kw['order'] != None else 'ACDEFGHIKLMNPQRSTVWY'

	myCodons = {
		'A': 4,
		'C': 2,
		'D': 2,
		'E': 2,
		'F': 2,
		'G': 4,
		'H': 2,
		'I': 3,
		'K': 2,
		'L': 6,
		'M': 1,
		'N': 2,
		'P': 4,
		'Q': 2,
		'R': 6,
		'S': 6,
		'T': 4,
		'V': 4,
		'W': 1,
		'Y': 2
	}

	encodings = []
	diPeptides = [aa1 + aa2 for aa1 in AA for aa2 in AA]
	header = ['#'] + diPeptides
	encodings.append(header)

	myTM = []
	for pair in diPeptides:
		myTM.append((myCodons[pair[0]] / 61) * (myCodons[pair[1]] / 61))

	AADict = {}
	for i in range(len(AA)):
		AADict[AA[i]] = i

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		tmpCode = [0] * 400
		for j in range(len(sequence) - 2 + 1):
			tmpCode[AADict[sequence[j]] * 20 + AADict[sequence[j+1]]] = tmpCode[AADict[sequence[j]] * 20 + AADict[sequence[j+1]]] +1
		if sum(tmpCode) != 0:
			tmpCode = [i/sum(tmpCode) for i in tmpCode]

		myTV = []
		for j in range(len(myTM)):
			myTV.append(myTM[j] * (1-myTM[j]) / (len(sequence) - 1))

		for j in range(len(tmpCode)):
			tmpCode[j] = (tmpCode[j] - myTM[j]) / math.sqrt(myTV[j])

		code = code + tmpCode
		encodings.append(code)
	return encodings


#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Authors: Chris (chris@nohup.cc) & Young (young@nohup.cc)

import sys
import numpy as np
import math
import re
import fileinput


def average(matrixSum, seqLen):
    # average the summary of rows
    matrix_array = np.array(matrixSum)
    matrix_array = np.divide(matrix_array, seqLen)
    matrix_array_shp = np.shape(matrix_array)
    matrix_average = [(np.reshape(matrix_array, (matrix_array_shp[0] * matrix_array_shp[1], )))]
    return matrix_average

def normalizePSSM(PSSM):
    PSSM=PSSM[:,1:21]
    PSSM=PSSM.astype(float)
    PSSM=np.array(PSSM)
    seq_cn=np.shape(PSSM)[0]
    PSSM_norm=[ [0.0] * 20 ] * seq_cn
    PSSM_norm=np.array(PSSM_norm)
    mean_matrix=np.mean(PSSM, axis=1)
    std_matrix=np.std(PSSM, axis=1)

    for i in range(seq_cn):
        for j in range(20):
            if std_matrix[i]==0.0:
                PSSM_norm[i][j] = PSSM[i][j]-mean_matrix[i]
            else:
                PSSM_norm[i][j]=(PSSM[i][j]-mean_matrix[i])/std_matrix[i]
    return PSSM_norm

def window(PSSM, w_smth, w_slide):
    # 0-19 represents amino acid 'ARNDCQEGHILKMFPSTWYV'
    w_smth=int(w_smth)
    w_slide=int(w_slide)
    Amino_vec = "ARNDCQEGHILKMFPSTWYV"

    PSSM=PSSM[:,1:21]
    PSSM=PSSM.astype(float)

    seq_cn = np.shape(PSSM)[0]

    #original PSSM
    PSSM_smth = np.array([[0.0]*20]*seq_cn)
    PSSM_orig = np.array(PSSM)
    #print PSSM_orig
    #section for PSSM_smth features
    PSSM_smth_full=pssm_smth(PSSM_orig,PSSM_smth,w_smth,seq_cn)
    #print PSSM_smth_full
    PSSM_smth_final=[[0.0]*20]*w_slide
    PSSM_smth_final=np.array(PSSM_smth_final)
    for i in range(w_slide):
        PSSM_smth_final[i]=PSSM_smth_full[i]
    matrix_final=average(PSSM_smth_final,1.0)
    return matrix_final

def pssm_smth(PSSM_orig, PSSM_smth, w_smth, l):
    for i in range(l):
        if i <(w_smth-1)/2:
            for j in range(i+int((w_smth-1)/2)+1):
                PSSM_smth[i]+=PSSM_orig[j]
        elif i>=(l-(w_smth-1)/2):
            for j in range(i-int((w_smth-1)/2),l):
                PSSM_smth[i]+=PSSM_orig[j]
        else:
            for j in range(i-int((w_smth-1)/2),i+int((w_smth-1)/2)+1):
                PSSM_smth[i]+=PSSM_orig[j]
    return PSSM_smth

def handleRows(PSSM, SWITCH, COUNT):
    '''
    if SWITCH=0, we filter no element.
    if SWITCH=1, we filter all the negative elements.
    if SWITCH=2, we filter all the negative and positive elements greater than expected.
    '''
    '''
    if COUNT=20, we generate a 20-dimension vector.
    if COUNT=400, we generate a 400-dimension vector.
    '''
    # 0-19 represents amino acid 'ARNDCQEGHILKMFPSTWYV'
    Amino_vec = "ARNDCQEGHILKMFPSTWYV"
    c = COUNT/20
    matrix_final = [ [0.0] * 20 ] * int(c)
    matrix_final=np.array(matrix_final)
 
    seq_cn = 0

    PSSM_shape=np.shape(PSSM)
    for i in range(PSSM_shape[0]):
        seq_cn += 1
        str_vec=PSSM[i]
        str_vec_positive=list(map(int, str_vec[1:21]))
        str_vec_positive=np.array(str_vec_positive)
        if SWITCH==1:
            str_vec_positive[str_vec_positive<0]=0
        elif SWITCH==2:
            str_vec_positive[str_vec_positive<0]=0
            str_vec_positive[str_vec_positive>7]=0
        #print "str_vec_positive="
        #print str_vec_positive
        if COUNT==20:
            
            matrix_final[0]=list(map(sum, list(zip(str_vec_positive, matrix_final[0]))))
        elif COUNT==400:
            
            matrix_final[Amino_vec.index(str_vec[0])] = list(map(sum, list(zip(str_vec_positive, matrix_final[Amino_vec.index(str_vec[0])]))))

        #print "matrix_final="
        #print matrix_final

    return matrix_final
    
def preHandleColumns(PSSM,STEP,PART,ID):
    '''
    if STEP=k, we calculate the relation betweem one residue and the kth residue afterward.
    '''
    '''
    if PART=0, we calculate the left part of PSSM.
    if PART=1, we calculate the right part of PSSM.
    '''
    '''
    if ID=0, we product the residue-pair.
    if ID=1, we minus the residue-pair.
    '''
    '''
    if KEY=1, we divide each element by the sum of elements in its column.
    if KEY=0, we don't perform the above process.
    '''
    if PART==0:
        #print "PART=",PART
        PSSM=PSSM[:,1:21]
    elif PART==1:
        #print "PART=",PART
        PSSM=PSSM[:, 21:]
    PSSM=PSSM.astype(float)
    matrix_final = [ [0.0] * 20 ] * 20
    matrix_final=np.array(matrix_final)
    seq_cn=np.shape(PSSM)[0]
    #print "seq_cn=",seq_cn

    if ID==0:
        #print "ID=",ID
        for i in range(20):
            for j in range(20):
                for k in range(seq_cn - STEP):
                    matrix_final[i][j]+=(PSSM[k][i]*PSSM[k+STEP][j])

    elif ID==1:
        #print "ID=",ID
        for i in range(20):
            for j in range(20):
                for k in range(seq_cn - STEP):
                    matrix_final[i][j] += ((PSSM[k][i]-PSSM[k+STEP][j]) * (PSSM[k][i]-PSSM[k+STEP][j])/4.0)
    #print matrix_final
    return matrix_final

def handleTriColumns(PSSM):
    matrix_final=[ [ [0.0] * 20] * 20] * 20
    matrix_final=np.array(matrix_final)
    PSSM=PSSM[:, 21:]
    PSSM=PSSM.astype(float)
    PSSM=np.array(PSSM)
    #print "PSSM="
    #print PSSM
    seq_cn=np.shape(PSSM)[0]
    for m in range(20):
        for n in range(20):
            for r in range(20):
                for i in range(seq_cn-2):
                    matrix_final[m][n][r]+=(PSSM[i][m]*PSSM[i+1][n]*PSSM[i+2][r])
    #print "matrix_final="
    #print matrix_final
    matrix_final=np.divide(matrix_final,1000000.0)
    matrix_final_shape=np.shape(matrix_final)
    #print matrix_final
    matrix_result = [(np.reshape(matrix_final, (matrix_final_shape[0] * matrix_final_shape[1] * matrix_final_shape[2], )))]
    return matrix_result

def handleMixed(PSSM,ALPHA):
    row1=[0.0] * 20
    row2=[0.0] * 20

    matrix_final = [ [0.0] * 40 ] * 1
    row1=np.array(row1)
    row2=np.array(row2)
    matrix_final=np.array(matrix_final)

    PSSM_norm=normalizePSSM(PSSM)
    seq_cn=np.shape(PSSM)[0]
    for i in range(seq_cn):
        #print PSSM_norm[i]
        row1=list(map(sum, list(zip(row1, PSSM_norm[i]))))
    #print row1
    row1=np.divide(row1,seq_cn)

    for j in range(20):
        for i in range(seq_cn-ALPHA):
            row2[j]+=(PSSM_norm[i][j]-PSSM_norm[i+ALPHA][j])*(PSSM_norm[i][j]-PSSM_norm[i+ALPHA][j])
    #print row2
    row2=np.divide(row2,seq_cn-ALPHA)

    row=np.hstack((row1,row2))
    matrix_final[0]=row
    #print np.shape(matrix_final)
    return matrix_final

def handleMixed2(PSSM,ALPHA):
    row1=[0.0] * 40
    row2=[ [0.0] * (2*ALPHA) ]*20
    matrix_final = [ [0.0] * (40+40*ALPHA) ] * 1

    row1=np.array(row1)
    row2=np.array(row2)
    matrix_final=np.array(matrix_final)

    PSSM_norm=normalizePSSM(PSSM)
    seq_cn=np.shape(PSSM)[0]
    for j in range(20):
        positive_count_1=0
        negative_count_1=0
        for i in range(seq_cn):
            if PSSM_norm[i][j]>=0:

                positive_count_1+=1
                row1[2*j]+=PSSM_norm[i][j]
            elif PSSM_norm[i][j]<0:

                negative_count_1+=1
                row1[2*j+1]+=PSSM_norm[i][j]
        #print "positive_count_1="
        #print positive_count_1
        #print "negative_count_1="
        #print negative_count_1
        row1[2*j]=row1[2*j]/positive_count_1
        row1[2*j+1]=row1[2*j+1]/negative_count_1

    for j in range(20):
        for alpha in range(1,ALPHA+1):
            positive_count_2=0
            negative_count_2=0
            for i in range(seq_cn-alpha):
                if (PSSM_norm[i][j]-PSSM_norm[i+alpha][j])>=0:
                    positive_count_2+=1
                    row2[j][2*alpha-2]+=(PSSM_norm[i][j]-PSSM_norm[i+alpha][j])*(PSSM_norm[i][j]-PSSM_norm[i+alpha][j])
                elif (PSSM_norm[i][j]-PSSM_norm[i+alpha][j])<0:
                    negative_count_2+=1
                    row2[j][2*alpha-1]+=(PSSM_norm[i][j]-PSSM_norm[i+alpha][j])*(PSSM_norm[i][j]-PSSM_norm[i+alpha][j])
            row2[j][2*alpha-2]=row2[j][2*alpha-2]/positive_count_2
            row2[j][2*alpha-1]=row2[j][2*alpha-1]/negative_count_2

    row2=average(row2,1.0)
    row=np.hstack((row1,row2[0]))
    matrix_final[0]=row
    #print np.shape(matrix_final)
    return matrix_final

def handleMixed3(PSSM):
    row=[ [0.0] * 110 ]*1
    row1=[0.0] * 100
    row2=[0.0] * 10
    row=np.array(row)
    row1=np.array(row1)
    row2=np.array(row2, dtype='float')

    seq_cn=np.shape(PSSM)[0]
    Amino_vec = "ARNDCQEGHILKMFPSTWYV"
    RPSSM=[ [0.0]*10 ]*seq_cn
    RPSSM=np.array(RPSSM)

    PSSM=PSSM[:,1:21]
    PSSM=PSSM.astype(float)
    PSSM=np.array(PSSM)
    #print "PSSM="
    #print PSSM
    RPSSM[:,0]=np.divide(list(map(sum,list(zip(PSSM[:,13],PSSM[:,17],PSSM[:,18])))), 3.0)
    RPSSM[:,1]=np.divide(list(map(sum,list(zip(PSSM[:,10],PSSM[:,12])))), 2.0)
    RPSSM[:,2]=np.divide(list(map(sum,list(zip(PSSM[:,9],PSSM[:,19])))), 2.0)
    RPSSM[:,3]=np.divide(list(map(sum,list(zip(PSSM[:,0],PSSM[:,15],PSSM[:,16])))), 3.0)
    RPSSM[:,4]=np.divide(list(map(sum,list(zip(PSSM[:,2],PSSM[:,8])))), 2.0)
    RPSSM[:,5]=np.divide(list(map(sum,list(zip(PSSM[:,5],PSSM[:,6],PSSM[:,3])))), 3.0)
    RPSSM[:,6]=np.divide(list(map(sum,list(zip(PSSM[:,1],PSSM[:,11])))), 2.0)
    RPSSM[:,7]=PSSM[:,4]
    RPSSM[:,8]=PSSM[:,7]
    RPSSM[:,9]=PSSM[:,14]
    # print "RPSSM="
    # print RPSSM
    mean_matrix=np.mean(RPSSM, axis=0)
    # print "mean_matrix="
    # print mean_matrix
    for j in range(10):
        for i in range(seq_cn):
            row2[j]+=(RPSSM[i][j]-mean_matrix[j])*(RPSSM[i][j]-mean_matrix[j])
    #print "row2="
    #print row2
    row2=np.divide(row2,seq_cn)
    matrix_final = [ [0.0] * 10 ] * 10
    matrix_final=np.array(matrix_final)
    for i in range(10):
        for j in range(10):
            for k in range(seq_cn-1):
                matrix_final[i][j] += ((RPSSM[k][i]-RPSSM[k+1][j]) * (RPSSM[k][i]-RPSSM[k+1][j])/2.0)
    #print "matrix_final="
    #print matrix_final
    row1=average(matrix_final,seq_cn-1)[0]
    #print "row1="
    #print row1
    row[0]=np.hstack((row1,row2))
    return row

def correlation(PSSM,ID,GROUP):
    # 0-19 represents amino acid 'ARNDCQEGHILKMFPSTWYV'
    Amino_vec = "ARNDCQEGHILKMFPSTWYV"
    #GROUP=10
    PSSM=PSSM[:,1:21]
    PSSM=PSSM.astype(float)
    #print PSSM
    #section for PSSM_AC features
    seq_cn = np.shape(PSSM)[0]
    g=GROUP
    l=seq_cn
    if ID==0:
        matrix_final=pssm_ac_cal(PSSM,g,l)
    elif ID==1:
        matrix_final=pssm_cc_cal(PSSM,g,l)

    matrix_final =  average(matrix_final, l)
    return matrix_final

def pssm_ac_cal(PSSM, g, l):

    PSSM_AC = np.array([ [0.0] * 20 ] * g)

    for pg in range(g):
        l_g = l - pg - 1
        for pj in range(20):
            sum_jl = 0.0
            for i in range(l):
                sum_jl += PSSM[i][pj]
            sum_jl /= l

            pssm_acjg = 0.0
            for i in range(l_g):
                pssm_acjg +=  (PSSM[i][pj]-sum_jl) * (PSSM[i+pg+1][pj]-sum_jl)
            pssm_acjg /= l_g
            PSSM_AC[pg][pj] = pssm_acjg
    return PSSM_AC

def pssm_cc_cal(PSSM, g, l):
    PSSM_CC = np.array([ [0.0] * 380 ] * g)
    for pg in range(g):
        l_g = l - pg - 1
        for pj_1 in range(20):
            sum_jl_1 = 0.0
            for i in range(l):
                sum_jl_1 += PSSM[i][pj_1]
            sum_jl_1 /= l

            for pj_2 in range(20):
                if pj_2!=pj_1:
                    sum_jl_2 = 0.0
                    for i in range(l):
                        sum_jl_2 += PSSM[i][pj_2]
                    sum_jl_2 /= l
                    pssm_acjg = 0.0
                    for i in range(l_g):
                        pssm_acjg += (PSSM[i][pj_1]-sum_jl_1) * (PSSM[i+pg+1][pj_2]-sum_jl_2)
                    pssm_acjg /= l_g
                    if(pj_1<pj_2):
                        PSSM_CC[pg][19*pj_1+(pj_2-1)] = pssm_acjg
                    else:
                        PSSM_CC[pg][19*pj_1+(pj_2)] = pssm_acjg

    return PSSM_CC



#!/usr/bin/env python
#_*_coding:utf-8_*_

import numpy as np
import pandas as pd

binBox = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]

def CHI2(encodings, labelFile):
	features = encodings[0][1:]
	encodings = np.array(encodings)[1:]
	data = encodings[:, 1:]
	shape = data.shape
	data = np.reshape(data, shape[0] * shape[1])
	data = np.reshape([float(i) for i in data], shape)

	e = ''
	if shape[0] < 5 or shape[1] < 2:
		return 0, e

	with open(labelFile) as f:
		records = f.readlines()
	myDict = {}
	try:
		for i in records:
			array = i.rstrip().split() if i.strip() != '' else None
			myDict[array[0]] = int(array[1])
	except IndexError as e:
		return 0, e

	labels = []
	for i in encodings:
		labels.append(myDict.get(i[0], 0))

	dataShape = data.shape

	if dataShape[1] != len(features):
		print('Error: inconsistent data shape with feature number.')
		return 0, 'Error: inconsistent data shape with feature number.'
	if dataShape[0] != len(labels):
		print('Error: inconsistent data shape with sample number.')
		return 0, 'Error: inconsistent data shape with sample number.'

	sampleNumber = len(data)
	labelClass = set(labels)
	myFea = {}
	for i in range(len(features)):
		array = data[:, i]
		newArray = list(pd.cut(array, len(binBox), labels= binBox))
		binBoxClass = set(newArray)
		myObservation = {}
		for j in range(len(labels)):
			#print(labels[j], newArray[j])
			myObservation[str(labels[j]) + str(newArray[j])] = myObservation.get(str(labels[j]) + str(newArray[j]), 0) + 1

		myExpect = {}
		for j in labelClass:
			for k in binBox:
				myExpect[str(j) + str(k)] = labels.count(j) * newArray.count(k) / sampleNumber

		chiValue = 0
		for j in labelClass:
			for k in binBoxClass:
				chiValue = chiValue + pow(((myObservation.get(str(j) + str(k), 0)) - myExpect.get(str(j) + str(k), 0)), 2) / myExpect[str(j) + str(k)]
		myFea[features[i]] = chiValue

	res = []
	res.append(['feature', 'CHI-value'])
	for key in sorted(myFea.items(), key=lambda item:item[1], reverse=True):
		res.append([key[0], '{0:.3f}'.format(myFea[key[0]])])
	return res, e